/* Richard Liang
   June 7, 2003
   
   Code to set up a 3D percolation cluster.

*/

#ifndef THREEDHF_H
#define THREEDHF_H

#include <stdio.h>
#include <stdlib.h>


// A struct to hold our whole percolation cluster.
struct percCluster3D
{
  char ***adj;
  char ***x;
  char ***y;
  char ***z;
  unsigned int size;

  // Some info about the cluster itself.
  double p;

  // The random seed used in building the cluster,
  // as well as the trial number (i.e. if we were
  // making several percolation clusters using this
  // random seed, we could thus recreate it with 
  // this information).
  int rseed;
  int trialNo;
};

typedef struct percCluster3D percCluster3D_t;


// A struct to represent information about a certain cluster.
struct clusterInfo
{
  int clusterSize;
  int clusterNo;
};

typedef struct clusterInfo clusterInfo_t;


// A function to allocate a new percCluster3D, empty and 
// uninitialized.
percCluster3D_t* create_percCluster3D_t(int size);


// A function to dispose of a percolation cluster.
void dispose_percCluster3D_t(percCluster3D_t* cluster);


// A function to actually set up a percolation cluster.
// Returns information about the largest cluster, and sets up
// the cluster to hold all the proper information.
// PRE:  cluster must be already built to the correct size.
//       random seed must already be initialized to the value
//        specified in cluster, and (trialNo-1)calls to doPerc 
//        have been performed beforehand
clusterInfo_t doPerc(percCluster3D_t* cluster, double p,
		     int rseed, int trialNo);


#endif
