/* Richard Liang
   A driver for our percolation function.

   June 7, 2003
*/

#include <stdio.h>
#include <stdlib.h>
#include "3dhf.h"
#include "showCluster.h"



int main(int argc, char **argv)
{
  // Counters, etc.
  int i,j,k;
  int size;

  percCluster3D_t* myCluster;
  double p;
  int rseed;

  clusterInfo_t myInfo;


  // Get some parameters from the command line.
  if (argc < 4)
    {
      fprintf(stderr, "Not enough parameters.\n");
      return -1;
    }
  
  // We have to get the size.
  size = atoi(argv[1]);
  if (0 >= size)
    {
      fprintf(stderr, "Size must be greater than 0.\n");
      return -1;
    }

  p = atof(argv[2]);
  if (p < 0 || p > 1)
    {
      fprintf(stderr, "Probability must be between 0 and 1 ");
      fprintf(stderr, "inclusive.\n");
      return -1;
    }

  rseed = atoi(argv[3]);
  if (0 > rseed)
    {
      fprintf(stderr, "Random seed must be >= 0.\n");
      return -1;
    }

  // Simply create a percolation cluster and ensure that all
  // is well.
  printf("Creating a percolation cluster of size %d.\n",size);

  myCluster = create_percCluster3D_t(size);

  // Now go through and check that everything is okay.
  printf("Accessing the arrays within the percolation cluster.\n");
  printf("No segfaults = good.\n");
  
  printf("Going through adj.\n");
  for (i=0; i<size; i++)
    {
      for (j=0; j<size; j++)
	{
	  for (k=0; k<size; k++)
	    {
	      (myCluster->adj)[i][j][k] = 0;
	    }
	}
    }


  printf("Going through x.\n");
  for (i=0; i<size-1; i++)
    {
      for (j=0; j<size; j++)
	{
	  for (k=0; k<size; k++)
	    {
	      (myCluster->x)[i][j][k] = 0;
	    }
	}
    }

  printf("Going through y.\n");
  for (i=0; i<size; i++)
    {
      for (j=0; j<(size-1); j++)
	{
	  for (k=0; k<size; k++)
	    {
	      (myCluster->y)[i][j][k] = 0;
	    }
	}
    }

  printf("Going through z.\n");
  for (i=0; i<size; i++)
    {
      for (j=0; j<size; j++)
	{
	  for (k=0; k<(size-1); k++)
	    {
	      (myCluster->z)[i][j][k] = 0;
	    }
	}
    }


  // Check the other stuff in the thing.
  printf("size==%u, p==%f, rseed==%d, trialNo==%d\n",
	 myCluster->size, myCluster->p, myCluster->rseed, 
	 myCluster->trialNo);


  // Check that doPerc doesn't have some stupid errors or
  // segfaults.
  printf("Doing percolation with p = %f, rseed = %d, and ",
	 p, rseed);
  printf("trialNo = 0.\n");
  srandom(rseed);

  myInfo = doPerc(myCluster, p, rseed, 0);
  
  printf("Max cluster number = %d, max cluster size = %d\n",
	 myInfo.clusterNo, myInfo.clusterSize);


  // We send the cluster to a drawing routine.
  drawCluster(myCluster, &argc, argv);

  // Dispose of the whole thing.
  printf("Destroying the cluster.\n");
  dispose_percCluster3D_t(myCluster);

  return 0;
}
