/* Richard Liang
   June 18, 2003
   Driver for our random walk program.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "walk_functions.h"
#include "show_walk.h"

int main(int argc, char **argv)
{
  // Counters, etc.
  //int i;
  int success;
  
  // We make a walk.
  walk_t myWalk;
  int numSteps = -1;
  int rseed = -1;
  int trialNo = 0;

  // Get some parameters.
  if (3 > argc)
    {
      fprintf(stderr, "Not enough parameters.\n");
      return -1;
    }

  numSteps = atoi(argv[1]);
  if (0 >= numSteps)
    {
      fprintf(stderr, "numSteps must be greater than 0.\n");
      return -1;
    }

  rseed = atoi(argv[2]);
  if (0 > rseed)
    {
      fprintf(stderr, "rseed must be greater than 0.\n");
      return -1;
    }

  // Allocate the array for the walk.
  myWalk.steps = (char*)malloc(numSteps * sizeof(char));
  memset(myWalk.steps, 0, numSteps * sizeof(char));

  printf("Performing a walk with numSteps==%d, rseed==%d, trialNo==%d\n",
	 numSteps, rseed, trialNo);
  
  srandom(rseed);

  success = doWalk(&myWalk, numSteps, rseed, trialNo);

  printf("doWalk returned %d.\n", success);

  // Analyze the contents of the walk.
  printf("The walk now has info:\n");
  printf("numSteps = %u\n", myWalk.numSteps);
  printf("rseed = %u\n", myWalk.rseed);
  printf("trialNo = %u\n", myWalk.trialNo);
  printf("maxSize = %u\n", myWalk.maxSize);
  
  //printf("The walk looks like:\n");
  //for (i=0; i<myWalk.numSteps; i++)
  //  {
  //    printf("%d\n", (myWalk.steps)[i]);
  //  }

  // Now we show the walk.
  drawWalk(&myWalk, &argc, argv);

  // De-allocate the memory we set aside for the walk.
  free(myWalk.steps);
  return 0;
}
