/* Richard Liang
   June 18, 2003
   Functions to generate a random walk in 3D.
*/

#include <stdio.h>
#include <stdlib.h>
#include "walk_functions.h"

#define dran() ((double) random()/(double) 0x7fffffff)

// Values we will need over and over. 
double posX = (double)1/(double)6;
double negX = (double)1/(double)6;
double posY = (double)1/(double)6;
double negY = (double)1/(double)6;
double posZ = (double)1/(double)6;
double negZ = (double)1/(double)6;

// Function to generate the walk.
// Note that the way we represent the walk is as follows:
//  1 -> positive X
// -1 -> negative X
//  2 -> positive Y
// -2 -> negative Y
//  3 -> positive Z
// -3 -> negative Z
// PRE & POST: see rand_functions.h
int doWalk(walk_t *myWalk, unsigned int numSteps, 
	   unsigned int rseed, unsigned int trialNo)
{
  // Counters, etc.
  unsigned int i;
  double currentRandom;

  unsigned int currentX;
  unsigned int currentY;
  unsigned int currentZ;

  unsigned int farthest;

  // First we stick all the information into myWalk.
  myWalk->numSteps = numSteps;
  myWalk->rseed = rseed;
  myWalk->trialNo = trialNo;

  // Set ourselves at the origin.
  currentX = currentY = currentZ = 0;
  farthest = 0;

  // Now we walk.
  for (i=0; i<numSteps; i++)
    {
      currentRandom = dran();
      
      if (currentRandom < posX)
	{
	  // We step in the positive X direction.
	  currentX++;
	  (myWalk->steps)[i] = 1;
	}
      else if (currentRandom < posX+negX)
	{
	  // We step in the negative X direction.
	  currentX--;
	  (myWalk->steps)[i] = -1;
	}
      else if (currentRandom < posX+negX+posY)
	{
	  currentY++;
	  (myWalk->steps)[i] = 2;
	}
      else if (currentRandom < posX+negX+posY+negY)
	{
	  currentY--;
	  (myWalk->steps)[i] = -2;
	}
      else if (currentRandom < posX+negX+posY+negY+posZ)
	{
	  currentZ++;
	  (myWalk->steps)[i] = 3;
	}
      //else if (currentRandom < posX+negX+posY+negY+posZ+negZ)
      else if (currentRandom < posX+negX+posY+negY+posZ+negZ)
	{
	  currentZ--;
	  (myWalk->steps)[i] = -3;
	}
      else
	{
	  // Something goofed in dran, or our buckets are set
	  // up wrong.
	  fprintf(stderr, "Error: currentRandom==%f\n",
		  currentRandom);
	  return -1;
	}

      if (abs(currentX) > farthest)
	farthest = abs(currentX);
      
      if (abs(currentY) > farthest)
	farthest = abs(currentY);
      
      if (abs(currentZ) > farthest)
	farthest = abs(currentZ);
    }

  // We're done.
  myWalk->maxSize = farthest;

  return 0;
}
